% Demo: Long Gap Interpolation:
% Audio interpolation in sub-bands: model-based extrapolation 
% Instead of zero-excitation, a time-reversed one is used in the
% highest bands

% The demostration given here is related to the paper P. A. A. Esquef and
% L. W. P. Biscainho, "An Efficient Model-Based Method for Reconstruction
% of Audio Signals across  Long Gaps", IEEE Trans. Speech and Language 
% Processing, vol. 14, no. 4, pp. 1391-1400, 2006. 
% Link: http://ieeexplore.ieee.org/iel5/9346/29688/01350530.pdf?arnumber=1350530 

% In this demo, a passage of an audio wave file containing a single gap is processed. 
% The gap location is known beforehand.
clear all; close all; clc
% One example
% [sig,fs]=wavread('test_signal_1');
% ind_ini=31970-200; % index of the first sample of the gap (minus 200 samples)
% ind_end=32990+200; % index of the last sample of the gap (plus 200 samples)
% The signal gap lasts about 1000 samples at 44.1 kHz 

% Another signal  
[sig,fs]=wavread('test_signal_2');
ind_ini=15130; % index of the first sample of the gap
ind_end=16015; % index of the last sample of the gap
% The signal gap lasts about 1000 samples at 44.1 kHz 

% Test signal 3 with way longer gap (4100 samples at 44.1 kHz)
% Test signal 3 is quite stationary, and that is why the long-durantion interpolation
% works
% [sig,fs]=wavread('test_signal_3');
% ind_ini=48685; % index of the first sample of the gap
% ind_end=52785; % index of the last sample of the gap

lmc=round(1.5*(ind_end+1-ind_ini)); % number of samples that precedes and 
% succedes the gap. These samples will be used for model parameter estimation. 
% lmc should be at least the length of the gap. Smaller lmc may work as
% well. Typical values are lmc equal to 1.5 to 3 times the length of the
% gap

%%%%%%%%%%%%%%%%%%%%%%%%%%%
[sigi]=LG_interp_WLDS(sig,ind_ini,ind_end,lmc,16); % Main function that performs long gap interpolation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%
% RESULTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Press a key to Listen to the Corrupted Signal')
pause
wavplay(sig,fs,'sync')

disp('Press a key to Listen to the Interpolated Signal')
pause
wavplay(sigi,fs,'sync')

disp('Press a key to See a plot that compares the original and reconstructed signal')
pause

plot(sigi,'r','linewidth',2); hold on; plot(sig,'b','linewidth',2); 
plot([ind_ini ind_ini],[-1 1],'k--')
plot([ind_end ind_end],[-1 1],'k--')
set(gca,'fontsize',16)
set(gcf,'position',[298 221 1005  734])

legend('Reconstructed','Corrupted','Gap Delimiters')
xlabel('Time (samples)')
ylabel('Amplitude')

